# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Krino clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Krino requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Krino build
## ---------------------------------------------------------------------------

set(Krino_CXX_COMPILER "/usr/bin/mpicxx")

set(Krino_C_COMPILER "/usr/bin/mpicc")

set(Krino_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(Krino_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Krino build
## ---------------------------------------------------------------------------

## Give the build type
set(Krino_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(Krino_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(Krino_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(Krino_Fortran_FLAGS [[ -O3]])
# Deprecated
set(Krino_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(Krino_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Krino_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(Krino_BUILD_SHARED_LIBS "ON")

set(Krino_LINKER /usr/bin/ld)
set(Krino_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Krino_INSTALL_DIR "/usr")

## List of package libraries
set(Krino_LIBRARIES Krino::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Krino_MPI_LIBRARIES "")
set(Krino_MPI_LIBRARY_DIRS "")
set(Krino_MPI_INCLUDE_DIRS "")
set(Krino_MPI_EXEC "/usr/bin/mpiexec")
set(Krino_MPI_EXEC_MAX_NUMPROCS "4")
set(Krino_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Krino_ENABLE_SEACASIoss ON)
set(Krino_ENABLE_SEACASExodus ON)
set(Krino_ENABLE_STKBalance ON)
set(Krino_ENABLE_STKExprEval ON)
set(Krino_ENABLE_STKMath ON)
set(Krino_ENABLE_STKIO ON)
set(Krino_ENABLE_STKSearch ON)
set(Krino_ENABLE_STKTopology ON)
set(Krino_ENABLE_STKUtil ON)
set(Krino_ENABLE_STKTools ON)
set(Krino_ENABLE_STKEmend ON)
set(Krino_ENABLE_Intrepid2 ON)
set(Krino_ENABLE_Boost ON)
set(Krino_ENABLE_MPI ON)
set(Krino_ENABLE_ParMETIS OFF)
set(Krino_ENABLE_yamlcpp OFF)

# Exported cache variables
set(Krino_ENABLE_DEBUG "OFF")
set(HAVE_KRINO_DEBUG "OFF")
set(TPL_ENABLE_yamlcpp "")
set(KRINO_HAVE_YAML "OFF")
set(TPL_ENABLE_ParMETIS "")
set(KRINO_HAVE_PARMETIS "OFF")

# Include configuration of dependent packages
if (NOT TARGET SEACASIoss::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASIoss/SEACASIossConfig.cmake")
endif()
if (NOT TARGET SEACASExodus::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASExodus/SEACASExodusConfig.cmake")
endif()
if (NOT TARGET STKBalance::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKBalance/STKBalanceConfig.cmake")
endif()
if (NOT TARGET STKExprEval::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKExprEval/STKExprEvalConfig.cmake")
endif()
if (NOT TARGET STKMath::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKMath/STKMathConfig.cmake")
endif()
if (NOT TARGET STKIO::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKIO/STKIOConfig.cmake")
endif()
if (NOT TARGET STKSearch::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKSearch/STKSearchConfig.cmake")
endif()
if (NOT TARGET STKTopology::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTopology/STKTopologyConfig.cmake")
endif()
if (NOT TARGET STKUtil::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKUtil/STKUtilConfig.cmake")
endif()
if (NOT TARGET STKTools::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTools/STKToolsConfig.cmake")
endif()
if (NOT TARGET STKEmend::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKEmend/STKEmendConfig.cmake")
endif()
if (NOT TARGET Intrepid2::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Intrepid2/Intrepid2Config.cmake")
endif()
if (NOT TARGET Boost::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/Boost/BoostConfig.cmake")
endif()
if (NOT TARGET MPI::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/MPI/MPIConfig.cmake")
endif()

# Import Krino targets
include("${CMAKE_CURRENT_LIST_DIR}/KrinoTargets.cmake")

# Standard TriBITS-compliant external package variables
set(Krino_IS_TRIBITS_COMPLIANT TRUE)
set(Krino_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(Krino_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Krino_EXPORTED_PACKAGE_LIBS_NAMES "krino_diagwriter_lib;krino_master_element_lib;krino_geometry_lib;krino_math_utils_lib;krino_surface_lib;krino_mesh_surface_lib;krino_quality_metric_lib;krino_mesh_utils_lib;krino_refinement_lib;krino_lib;krino_region_lib;krino_rebalance_utils_lib;krino_parser_lib")

foreach(libname IN LISTS Krino_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Krino::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Krino::${libname}', or better yet,"
      " 'Krino::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Krino'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Krino_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
