# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Panzer clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Panzer requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Panzer build
## ---------------------------------------------------------------------------

set(Panzer_CXX_COMPILER "/usr/bin/mpicxx")

set(Panzer_C_COMPILER "/usr/bin/mpicc")

set(Panzer_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(Panzer_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Panzer build
## ---------------------------------------------------------------------------

## Give the build type
set(Panzer_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(Panzer_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(Panzer_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(Panzer_Fortran_FLAGS [[ -O3]])
# Deprecated
set(Panzer_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(Panzer_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Panzer_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(Panzer_BUILD_SHARED_LIBS "ON")

set(Panzer_LINKER /usr/bin/ld)
set(Panzer_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Panzer_INSTALL_DIR "/usr")

## List of package libraries
set(Panzer_LIBRARIES Panzer::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Panzer_MPI_LIBRARIES "")
set(Panzer_MPI_LIBRARY_DIRS "")
set(Panzer_MPI_INCLUDE_DIRS "")
set(Panzer_MPI_EXEC "/usr/bin/mpiexec")
set(Panzer_MPI_EXEC_MAX_NUMPROCS "4")
set(Panzer_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Panzer_ENABLE_PanzerCore ON)
set(Panzer_ENABLE_PanzerDofMgr ON)
set(Panzer_ENABLE_PanzerDiscFE ON)
set(Panzer_ENABLE_PanzerAdaptersSTK ON)
set(Panzer_ENABLE_PanzerMiniEM ON)
set(Panzer_ENABLE_PanzerExprEval OFF)

# Exported cache variables
set(Panzer_ENABLE_DEBUG "OFF")
set(PANZER_DEBUG "OFF")
set(Panzer_ENABLE_EXPLICIT_INSTANTIATION "ON")
set(HAVE_PANZER_EXPLICIT_INSTANTIATION "ON")

# Include configuration of dependent packages
if (NOT TARGET PanzerCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerCore/PanzerCoreConfig.cmake")
endif()
if (NOT TARGET PanzerDofMgr::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerDofMgr/PanzerDofMgrConfig.cmake")
endif()
if (NOT TARGET PanzerDiscFE::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerDiscFE/PanzerDiscFEConfig.cmake")
endif()
if (NOT TARGET PanzerAdaptersSTK::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerAdaptersSTK/PanzerAdaptersSTKConfig.cmake")
endif()
if (NOT TARGET PanzerMiniEM::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerMiniEM/PanzerMiniEMConfig.cmake")
endif()

# Import Panzer targets
include("${CMAKE_CURRENT_LIST_DIR}/PanzerTargets.cmake")

# Standard TriBITS-compliant external package variables
set(Panzer_IS_TRIBITS_COMPLIANT TRUE)
set(Panzer_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(Panzer_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Panzer_EXPORTED_PACKAGE_LIBS_NAMES "panzer-core;PanzerCore_all_libs;panzer-dof-mgr;PanzerDofMgr_all_libs;panzer-disc-fe;PanzerDiscFE_all_libs;panzer-stk;PanzerAdaptersSTK_all_libs;panzer-mini-fe;PanzerMiniEM_all_libs")

foreach(libname IN LISTS Panzer_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Panzer::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Panzer::${libname}', or better yet,"
      " 'Panzer::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Panzer'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Panzer_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
