# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/PanzerMiniEM clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "PanzerMiniEM requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/PanzerMiniEM build
## ---------------------------------------------------------------------------

set(PanzerMiniEM_CXX_COMPILER "/usr/bin/mpicxx")

set(PanzerMiniEM_C_COMPILER "/usr/bin/mpicc")

set(PanzerMiniEM_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(PanzerMiniEM_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/PanzerMiniEM build
## ---------------------------------------------------------------------------

## Give the build type
set(PanzerMiniEM_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(PanzerMiniEM_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(PanzerMiniEM_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(PanzerMiniEM_Fortran_FLAGS [[ -O3]])
# Deprecated
set(PanzerMiniEM_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(PanzerMiniEM_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(PanzerMiniEM_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(PanzerMiniEM_BUILD_SHARED_LIBS "ON")

set(PanzerMiniEM_LINKER /usr/bin/ld)
set(PanzerMiniEM_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(PanzerMiniEM_INSTALL_DIR "/usr")

## List of package libraries
set(PanzerMiniEM_LIBRARIES PanzerMiniEM::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(PanzerMiniEM_MPI_LIBRARIES "")
set(PanzerMiniEM_MPI_LIBRARY_DIRS "")
set(PanzerMiniEM_MPI_INCLUDE_DIRS "")
set(PanzerMiniEM_MPI_EXEC "/usr/bin/mpiexec")
set(PanzerMiniEM_MPI_EXEC_MAX_NUMPROCS "4")
set(PanzerMiniEM_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(PanzerMiniEM_ENABLE_PanzerCore ON)
set(PanzerMiniEM_ENABLE_PanzerDofMgr ON)
set(PanzerMiniEM_ENABLE_PanzerDiscFE ON)
set(PanzerMiniEM_ENABLE_PanzerAdaptersSTK ON)
set(PanzerMiniEM_ENABLE_Phalanx ON)
set(PanzerMiniEM_ENABLE_Belos ON)
set(PanzerMiniEM_ENABLE_Teko ON)
set(PanzerMiniEM_ENABLE_MueLu ON)
set(PanzerMiniEM_ENABLE_MPI ON)
set(PanzerMiniEM_ENABLE_ML ON)
set(PanzerMiniEM_ENABLE_ShyLU_NodeTacho ON)
set(PanzerMiniEM_ENABLE_PAPI OFF)
set(PanzerMiniEM_ENABLE_CAMAL OFF)

# Exported cache variables
set(Panzer_ENABLE_DEBUG "OFF")
set(PANZER_DEBUG "OFF")
set(Panzer_ENABLE_EXPLICIT_INSTANTIATION "ON")
set(HAVE_PANZER_EXPLICIT_INSTANTIATION "ON")

# Include configuration of dependent packages
if (NOT TARGET PanzerCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerCore/PanzerCoreConfig.cmake")
endif()
if (NOT TARGET PanzerDofMgr::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerDofMgr/PanzerDofMgrConfig.cmake")
endif()
if (NOT TARGET PanzerDiscFE::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerDiscFE/PanzerDiscFEConfig.cmake")
endif()
if (NOT TARGET PanzerAdaptersSTK::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../PanzerAdaptersSTK/PanzerAdaptersSTKConfig.cmake")
endif()
if (NOT TARGET Phalanx::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Phalanx/PhalanxConfig.cmake")
endif()
if (NOT TARGET Belos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Belos/BelosConfig.cmake")
endif()
if (NOT TARGET Teko::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Teko/TekoConfig.cmake")
endif()
if (NOT TARGET MueLu::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../MueLu/MueLuConfig.cmake")
endif()
if (NOT TARGET MPI::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/MPI/MPIConfig.cmake")
endif()
if (NOT TARGET ML::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ML/MLConfig.cmake")
endif()
if (NOT TARGET ShyLU_NodeTacho::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ShyLU_NodeTacho/ShyLU_NodeTachoConfig.cmake")
endif()

# Import PanzerMiniEM targets
include("${CMAKE_CURRENT_LIST_DIR}/PanzerMiniEMTargets.cmake")

# Standard TriBITS-compliant external package variables
set(PanzerMiniEM_IS_TRIBITS_COMPLIANT TRUE)
set(PanzerMiniEM_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(PanzerMiniEM_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(PanzerMiniEM_EXPORTED_PACKAGE_LIBS_NAMES "panzer-mini-fe")

foreach(libname IN LISTS PanzerMiniEM_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE PanzerMiniEM::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'PanzerMiniEM::${libname}', or better yet,"
      " 'PanzerMiniEM::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'PanzerMiniEM'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'PanzerMiniEM_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
