# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Pliris clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Pliris requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Pliris build
## ---------------------------------------------------------------------------

set(Pliris_CXX_COMPILER "/usr/bin/mpicxx")

set(Pliris_C_COMPILER "/usr/bin/mpicc")

set(Pliris_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(Pliris_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Pliris build
## ---------------------------------------------------------------------------

## Give the build type
set(Pliris_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(Pliris_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(Pliris_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(Pliris_Fortran_FLAGS [[ -O3]])
# Deprecated
set(Pliris_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(Pliris_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Pliris_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(Pliris_BUILD_SHARED_LIBS "ON")

set(Pliris_LINKER /usr/bin/ld)
set(Pliris_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Pliris_INSTALL_DIR "/usr")

## List of package libraries
set(Pliris_LIBRARIES Pliris::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Pliris_MPI_LIBRARIES "")
set(Pliris_MPI_LIBRARY_DIRS "")
set(Pliris_MPI_INCLUDE_DIRS "")
set(Pliris_MPI_EXEC "/usr/bin/mpiexec")
set(Pliris_MPI_EXEC_MAX_NUMPROCS "4")
set(Pliris_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Pliris_ENABLE_Epetra ON)

# Exported cache variables
set(Pliris_ENABLE_ZCPLX "OFF")
set(Pliris_ENABLE_SCPLX "OFF")
set(Pliris_ENABLE_SREAL "OFF")
set(Pliris_ENABLE_DREAL "ON")

# Include configuration of dependent packages
if (NOT TARGET Epetra::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Epetra/EpetraConfig.cmake")
endif()

# Import Pliris targets
include("${CMAKE_CURRENT_LIST_DIR}/PlirisTargets.cmake")

# Standard TriBITS-compliant external package variables
set(Pliris_IS_TRIBITS_COMPLIANT TRUE)
set(Pliris_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(Pliris_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Pliris_EXPORTED_PACKAGE_LIBS_NAMES "dpliris")

foreach(libname IN LISTS Pliris_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Pliris::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Pliris::${libname}', or better yet,"
      " 'Pliris::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Pliris'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Pliris_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
