# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/SEACASCpup clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "SEACASCpup requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/SEACASCpup build
## ---------------------------------------------------------------------------

set(SEACASCpup_CXX_COMPILER "/usr/bin/mpicxx")

set(SEACASCpup_C_COMPILER "/usr/bin/mpicc")

set(SEACASCpup_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(SEACASCpup_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/SEACASCpup build
## ---------------------------------------------------------------------------

## Give the build type
set(SEACASCpup_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(SEACASCpup_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(SEACASCpup_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(SEACASCpup_Fortran_FLAGS [[ -O3]])
# Deprecated
set(SEACASCpup_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(SEACASCpup_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(SEACASCpup_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(SEACASCpup_BUILD_SHARED_LIBS "ON")

set(SEACASCpup_LINKER /usr/bin/ld)
set(SEACASCpup_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(SEACASCpup_INSTALL_DIR "/usr")

## List of package libraries
set(SEACASCpup_LIBRARIES SEACASCpup::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(SEACASCpup_MPI_LIBRARIES "")
set(SEACASCpup_MPI_LIBRARY_DIRS "")
set(SEACASCpup_MPI_INCLUDE_DIRS "")
set(SEACASCpup_MPI_EXEC "/usr/bin/mpiexec")
set(SEACASCpup_MPI_EXEC_MAX_NUMPROCS "4")
set(SEACASCpup_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(SEACASCpup_ENABLE_SEACASExodus ON)
set(SEACASCpup_ENABLE_SEACASIoss ON)
set(SEACASCpup_ENABLE_SEACASSuplibC ON)
set(SEACASCpup_ENABLE_SEACASSuplibCpp ON)

# Exported cache variables
set(SEACAS_ENABLE_DEBUG "OFF")
set(HAVE_SEACAS_DEBUG "OFF")
set(SEACASExodus_ENABLE_THREADSAFE "OFF")
set(EXODUS_THREADSAFE "OFF")
set(SEACASExodus_ENABLE_STATIC "OFF")
set(EXODUS_STATIC "OFF")
set(SEACASExodus_ENABLE_SHARED "OFF")
set(EXODUS_SHARED "OFF")
set(SEACASIoss_ENABLE_THREADSAFE "OFF")
set(IOSS_THREADSAFE "OFF")

# Include configuration of dependent packages
if (NOT TARGET SEACASExodus::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASExodus/SEACASExodusConfig.cmake")
endif()
if (NOT TARGET SEACASIoss::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASIoss/SEACASIossConfig.cmake")
endif()
if (NOT TARGET SEACASSuplibC::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASSuplibC/SEACASSuplibCConfig.cmake")
endif()
if (NOT TARGET SEACASSuplibCpp::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASSuplibCpp/SEACASSuplibCppConfig.cmake")
endif()

# Import SEACASCpup targets
include("${CMAKE_CURRENT_LIST_DIR}/SEACASCpupTargets.cmake")

# Standard TriBITS-compliant external package variables
set(SEACASCpup_IS_TRIBITS_COMPLIANT TRUE)
set(SEACASCpup_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(SEACASCpup_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(SEACASCpup_EXPORTED_PACKAGE_LIBS_NAMES "")

foreach(libname IN LISTS SEACASCpup_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE SEACASCpup::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'SEACASCpup::${libname}', or better yet,"
      " 'SEACASCpup::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'SEACASCpup'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'SEACASCpup_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
