# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/SEACASIoss clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "SEACASIoss requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/SEACASIoss build
## ---------------------------------------------------------------------------

set(SEACASIoss_CXX_COMPILER "/usr/bin/mpicxx")

set(SEACASIoss_C_COMPILER "/usr/bin/mpicc")

set(SEACASIoss_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(SEACASIoss_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/SEACASIoss build
## ---------------------------------------------------------------------------

## Give the build type
set(SEACASIoss_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(SEACASIoss_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(SEACASIoss_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(SEACASIoss_Fortran_FLAGS [[ -O3]])
# Deprecated
set(SEACASIoss_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(SEACASIoss_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(SEACASIoss_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(SEACASIoss_BUILD_SHARED_LIBS "ON")

set(SEACASIoss_LINKER /usr/bin/ld)
set(SEACASIoss_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(SEACASIoss_INSTALL_DIR "/usr")

## List of package libraries
set(SEACASIoss_LIBRARIES SEACASIoss::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(SEACASIoss_MPI_LIBRARIES "")
set(SEACASIoss_MPI_LIBRARY_DIRS "")
set(SEACASIoss_MPI_INCLUDE_DIRS "")
set(SEACASIoss_MPI_EXEC "/usr/bin/mpiexec")
set(SEACASIoss_MPI_EXEC_MAX_NUMPROCS "4")
set(SEACASIoss_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(SEACASIoss_ENABLE_SEACASExodus ON)
set(SEACASIoss_ENABLE_Pamgen ON)
set(SEACASIoss_ENABLE_Zoltan ON)
set(SEACASIoss_ENABLE_Kokkos ON)
set(SEACASIoss_ENABLE_HDF5 ON)
set(SEACASIoss_ENABLE_CGNS OFF)
set(SEACASIoss_ENABLE_ParMETIS OFF)
set(SEACASIoss_ENABLE_Faodel OFF)
set(SEACASIoss_ENABLE_Cereal OFF)
set(SEACASIoss_ENABLE_DLlib ON)
set(SEACASIoss_ENABLE_Pthread OFF)
set(SEACASIoss_ENABLE_DataWarp OFF)
set(SEACASIoss_ENABLE_ADIOS2 OFF)
set(SEACASIoss_ENABLE_Catalyst2 OFF)

# Exported cache variables
set(SEACAS_ENABLE_DEBUG "OFF")
set(HAVE_SEACAS_DEBUG "OFF")
set(SEACASExodus_ENABLE_THREADSAFE "OFF")
set(EXODUS_THREADSAFE "OFF")
set(SEACASExodus_ENABLE_STATIC "OFF")
set(EXODUS_STATIC "OFF")
set(SEACASExodus_ENABLE_SHARED "OFF")
set(EXODUS_SHARED "OFF")
set(SEACASIoss_ENABLE_THREADSAFE "OFF")
set(IOSS_THREADSAFE "OFF")

# Include configuration of dependent packages
if (NOT TARGET SEACASExodus::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASExodus/SEACASExodusConfig.cmake")
endif()
if (NOT TARGET Pamgen::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Pamgen/PamgenConfig.cmake")
endif()
if (NOT TARGET Zoltan::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Zoltan/ZoltanConfig.cmake")
endif()
if (NOT TARGET Kokkos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Kokkos/KokkosConfig.cmake")
endif()
if (NOT TARGET HDF5::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/HDF5/HDF5Config.cmake")
endif()
if (NOT TARGET DLlib::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/DLlib/DLlibConfig.cmake")
endif()

# Import SEACASIoss targets
include("${CMAKE_CURRENT_LIST_DIR}/SEACASIossTargets.cmake")

# Standard TriBITS-compliant external package variables
set(SEACASIoss_IS_TRIBITS_COMPLIANT TRUE)
set(SEACASIoss_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(SEACASIoss_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(SEACASIoss_EXPORTED_PACKAGE_LIBS_NAMES "Ioex;Iopg;Ionull;Iovs;Iogn;Iotm;Iogs;Iohb;Iotr;Ionit;io_info_lib;Ioss")

foreach(libname IN LISTS SEACASIoss_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE SEACASIoss::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'SEACASIoss::${libname}', or better yet,"
      " 'SEACASIoss::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'SEACASIoss'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'SEACASIoss_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
