# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/STK clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "STK requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/STK build
## ---------------------------------------------------------------------------

set(STK_CXX_COMPILER "/usr/bin/mpicxx")

set(STK_C_COMPILER "/usr/bin/mpicc")

set(STK_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(STK_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/STK build
## ---------------------------------------------------------------------------

## Give the build type
set(STK_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(STK_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(STK_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(STK_Fortran_FLAGS [[ -O3]])
# Deprecated
set(STK_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(STK_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(STK_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(STK_BUILD_SHARED_LIBS "ON")

set(STK_LINKER /usr/bin/ld)
set(STK_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(STK_INSTALL_DIR "/usr")

## List of package libraries
set(STK_LIBRARIES STK::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(STK_MPI_LIBRARIES "")
set(STK_MPI_LIBRARY_DIRS "")
set(STK_MPI_INCLUDE_DIRS "")
set(STK_MPI_EXEC "/usr/bin/mpiexec")
set(STK_MPI_EXEC_MAX_NUMPROCS "4")
set(STK_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(STK_ENABLE_STKUtil ON)
set(STK_ENABLE_STKEmend ON)
set(STK_ENABLE_STKCoupling ON)
set(STK_ENABLE_STKMath ON)
set(STK_ENABLE_STKSimd ON)
set(STK_ENABLE_STKNGP_TEST OFF)
set(STK_ENABLE_STKExprEval ON)
set(STK_ENABLE_STKTopology ON)
set(STK_ENABLE_STKSearch ON)
set(STK_ENABLE_STKMiddle_mesh OFF)
set(STK_ENABLE_STKTransfer ON)
set(STK_ENABLE_STKMesh ON)
set(STK_ENABLE_STKSearchUtil ON)
set(STK_ENABLE_STKTransferUtil OFF)
set(STK_ENABLE_STKIO ON)
set(STK_ENABLE_STKMiddle_mesh_util OFF)
set(STK_ENABLE_STKTools ON)
set(STK_ENABLE_STKBalance ON)
set(STK_ENABLE_STKUnit_test_utils OFF)
set(STK_ENABLE_STKUnit_tests ON)
set(STK_ENABLE_STKDoc_tests ON)
set(STK_ENABLE_STKIntegration_tests ON)
set(STK_ENABLE_STKPerformance_tests ON)
set(STK_ENABLE_MPI ON)

# Exported cache variables
set(STK_ENABLE_DEBUG "OFF")
set(HAVE_STK_DEBUG "OFF")

# Include configuration of dependent packages
if (NOT TARGET STKUtil::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKUtil/STKUtilConfig.cmake")
endif()
if (NOT TARGET STKEmend::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKEmend/STKEmendConfig.cmake")
endif()
if (NOT TARGET STKCoupling::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKCoupling/STKCouplingConfig.cmake")
endif()
if (NOT TARGET STKMath::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKMath/STKMathConfig.cmake")
endif()
if (NOT TARGET STKSimd::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKSimd/STKSimdConfig.cmake")
endif()
if (NOT TARGET STKExprEval::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKExprEval/STKExprEvalConfig.cmake")
endif()
if (NOT TARGET STKTopology::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTopology/STKTopologyConfig.cmake")
endif()
if (NOT TARGET STKSearch::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKSearch/STKSearchConfig.cmake")
endif()
if (NOT TARGET STKTransfer::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTransfer/STKTransferConfig.cmake")
endif()
if (NOT TARGET STKMesh::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKMesh/STKMeshConfig.cmake")
endif()
if (NOT TARGET STKSearchUtil::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKSearchUtil/STKSearchUtilConfig.cmake")
endif()
if (NOT TARGET STKIO::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKIO/STKIOConfig.cmake")
endif()
if (NOT TARGET STKTools::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTools/STKToolsConfig.cmake")
endif()
if (NOT TARGET STKBalance::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKBalance/STKBalanceConfig.cmake")
endif()
if (NOT TARGET STKUnit_tests::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKUnit_tests/STKUnit_testsConfig.cmake")
endif()
if (NOT TARGET STKDoc_tests::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKDoc_tests/STKDoc_testsConfig.cmake")
endif()
if (NOT TARGET STKIntegration_tests::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKIntegration_tests/STKIntegration_testsConfig.cmake")
endif()
if (NOT TARGET STKPerformance_tests::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKPerformance_tests/STKPerformance_testsConfig.cmake")
endif()
if (NOT TARGET MPI::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/MPI/MPIConfig.cmake")
endif()

# Import STK targets
include("${CMAKE_CURRENT_LIST_DIR}/STKTargets.cmake")

# Standard TriBITS-compliant external package variables
set(STK_IS_TRIBITS_COMPLIANT TRUE)
set(STK_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(STK_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(STK_EXPORTED_PACKAGE_LIBS_NAMES "stk_util_util;stk_util_parallel;stk_util_env;stk_util_registry;stk_util_diag;stk_util_command_line;STKUtil_all_libs;stk_emend;STKEmend_all_libs;stk_coupling;STKCoupling_all_libs;stk_math;STKMath_all_libs;stk_simd;STKSimd_all_libs;stk_expreval;STKExprEval_all_libs;stk_topology;STKTopology_all_libs;stk_search;STKSearch_all_libs;stk_transfer;STKTransfer_all_libs;stk_mesh_base;STKMesh_all_libs;stk_search_util;STKSearchUtil_all_libs;stk_io_util;stk_io;STKIO_all_libs;stk_transfer_utils_lib;stk_tools_lib;stk_pmesh_lib;STKTools_all_libs;search_tolerance_algs;stk_balance_lib;STKBalance_all_libs;STKUnit_tests_all_libs;STKDoc_tests_all_libs;STKIntegration_tests_all_libs;STKPerformance_tests_all_libs")

foreach(libname IN LISTS STK_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE STK::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'STK::${libname}', or better yet,"
      " 'STK::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'STK'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'STK_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
