# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/STKMesh clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "STKMesh requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/STKMesh build
## ---------------------------------------------------------------------------

set(STKMesh_CXX_COMPILER "/usr/bin/mpicxx")

set(STKMesh_C_COMPILER "/usr/bin/mpicc")

set(STKMesh_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(STKMesh_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/STKMesh build
## ---------------------------------------------------------------------------

## Give the build type
set(STKMesh_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(STKMesh_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(STKMesh_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(STKMesh_Fortran_FLAGS [[ -O3]])
# Deprecated
set(STKMesh_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(STKMesh_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(STKMesh_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(STKMesh_BUILD_SHARED_LIBS "ON")

set(STKMesh_LINKER /usr/bin/ld)
set(STKMesh_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(STKMesh_INSTALL_DIR "/usr")

## List of package libraries
set(STKMesh_LIBRARIES STKMesh::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(STKMesh_MPI_LIBRARIES "")
set(STKMesh_MPI_LIBRARY_DIRS "")
set(STKMesh_MPI_INCLUDE_DIRS "")
set(STKMesh_MPI_EXEC "/usr/bin/mpiexec")
set(STKMesh_MPI_EXEC_MAX_NUMPROCS "4")
set(STKMesh_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(STKMesh_ENABLE_Shards ON)
set(STKMesh_ENABLE_STKTopology ON)
set(STKMesh_ENABLE_STKUtil ON)
set(STKMesh_ENABLE_Kokkos ON)
set(STKMesh_ENABLE_BLAS ON)
set(STKMesh_ENABLE_MPI ON)

# Exported cache variables
set(STK_ENABLE_DEBUG "OFF")
set(HAVE_STK_DEBUG "OFF")

# Include configuration of dependent packages
if (NOT TARGET Shards::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Shards/ShardsConfig.cmake")
endif()
if (NOT TARGET STKTopology::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTopology/STKTopologyConfig.cmake")
endif()
if (NOT TARGET STKUtil::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKUtil/STKUtilConfig.cmake")
endif()
if (NOT TARGET Kokkos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Kokkos/KokkosConfig.cmake")
endif()
if (NOT TARGET BLAS::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/BLAS/BLASConfig.cmake")
endif()
if (NOT TARGET MPI::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/MPI/MPIConfig.cmake")
endif()

# Import STKMesh targets
include("${CMAKE_CURRENT_LIST_DIR}/STKMeshTargets.cmake")

# Standard TriBITS-compliant external package variables
set(STKMesh_IS_TRIBITS_COMPLIANT TRUE)
set(STKMesh_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(STKMesh_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(STKMesh_EXPORTED_PACKAGE_LIBS_NAMES "stk_mesh_base")

foreach(libname IN LISTS STKMesh_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE STKMesh::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'STKMesh::${libname}', or better yet,"
      " 'STKMesh::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'STKMesh'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'STKMesh_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
