# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/STKTransfer clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "STKTransfer requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/STKTransfer build
## ---------------------------------------------------------------------------

set(STKTransfer_CXX_COMPILER "/usr/bin/mpicxx")

set(STKTransfer_C_COMPILER "/usr/bin/mpicc")

set(STKTransfer_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(STKTransfer_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/STKTransfer build
## ---------------------------------------------------------------------------

## Give the build type
set(STKTransfer_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(STKTransfer_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(STKTransfer_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(STKTransfer_Fortran_FLAGS [[ -O3]])
# Deprecated
set(STKTransfer_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(STKTransfer_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(STKTransfer_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(STKTransfer_BUILD_SHARED_LIBS "ON")

set(STKTransfer_LINKER /usr/bin/ld)
set(STKTransfer_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(STKTransfer_INSTALL_DIR "/usr")

## List of package libraries
set(STKTransfer_LIBRARIES STKTransfer::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(STKTransfer_MPI_LIBRARIES "")
set(STKTransfer_MPI_LIBRARY_DIRS "")
set(STKTransfer_MPI_INCLUDE_DIRS "")
set(STKTransfer_MPI_EXEC "/usr/bin/mpiexec")
set(STKTransfer_MPI_EXEC_MAX_NUMPROCS "4")
set(STKTransfer_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(STKTransfer_ENABLE_STKUtil ON)
set(STKTransfer_ENABLE_STKSearch ON)
set(STKTransfer_ENABLE_STKMiddle_mesh OFF)

# Exported cache variables
set(STK_ENABLE_DEBUG "OFF")
set(HAVE_STK_DEBUG "OFF")

# Include configuration of dependent packages
if (NOT TARGET STKUtil::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKUtil/STKUtilConfig.cmake")
endif()
if (NOT TARGET STKSearch::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKSearch/STKSearchConfig.cmake")
endif()

# Import STKTransfer targets
include("${CMAKE_CURRENT_LIST_DIR}/STKTransferTargets.cmake")

# Standard TriBITS-compliant external package variables
set(STKTransfer_IS_TRIBITS_COMPLIANT TRUE)
set(STKTransfer_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(STKTransfer_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(STKTransfer_EXPORTED_PACKAGE_LIBS_NAMES "stk_transfer")

foreach(libname IN LISTS STKTransfer_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE STKTransfer::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'STKTransfer::${libname}', or better yet,"
      " 'STKTransfer::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'STKTransfer'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'STKTransfer_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
