# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Stokhos clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Stokhos requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Stokhos build
## ---------------------------------------------------------------------------

set(Stokhos_CXX_COMPILER "/usr/bin/mpicxx")

set(Stokhos_C_COMPILER "/usr/bin/mpicc")

set(Stokhos_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(Stokhos_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Stokhos build
## ---------------------------------------------------------------------------

## Give the build type
set(Stokhos_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(Stokhos_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(Stokhos_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(Stokhos_Fortran_FLAGS [[ -O3]])
# Deprecated
set(Stokhos_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(Stokhos_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Stokhos_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(Stokhos_BUILD_SHARED_LIBS "ON")

set(Stokhos_LINKER /usr/bin/ld)
set(Stokhos_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Stokhos_INSTALL_DIR "/usr")

## List of package libraries
set(Stokhos_LIBRARIES Stokhos::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Stokhos_MPI_LIBRARIES "")
set(Stokhos_MPI_LIBRARY_DIRS "")
set(Stokhos_MPI_INCLUDE_DIRS "")
set(Stokhos_MPI_EXEC "/usr/bin/mpiexec")
set(Stokhos_MPI_EXEC_MAX_NUMPROCS "4")
set(Stokhos_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Stokhos_ENABLE_Teuchos ON)
set(Stokhos_ENABLE_Kokkos ON)
set(Stokhos_ENABLE_EpetraExt ON)
set(Stokhos_ENABLE_Ifpack ON)
set(Stokhos_ENABLE_ML ON)
set(Stokhos_ENABLE_Anasazi ON)
set(Stokhos_ENABLE_Sacado ON)
set(Stokhos_ENABLE_NOX ON)
set(Stokhos_ENABLE_Isorropia ON)
set(Stokhos_ENABLE_KokkosKernels ON)
set(Stokhos_ENABLE_TeuchosKokkosComm ON)
set(Stokhos_ENABLE_Tpetra ON)
set(Stokhos_ENABLE_Ifpack2 ON)
set(Stokhos_ENABLE_MueLu ON)
set(Stokhos_ENABLE_Belos ON)
set(Stokhos_ENABLE_Amesos2 ON)
set(Stokhos_ENABLE_Thyra ON)
set(Stokhos_ENABLE_Xpetra ON)
set(Stokhos_ENABLE_ForUQTK OFF)
set(Stokhos_ENABLE_CUDA OFF)
set(Stokhos_ENABLE_Thrust OFF)
set(Stokhos_ENABLE_Cusp OFF)
set(Stokhos_ENABLE_CUSPARSE OFF)
set(Stokhos_ENABLE_Clp OFF)
set(Stokhos_ENABLE_GLPK OFF)
set(Stokhos_ENABLE_qpOASES OFF)
set(Stokhos_ENABLE_Boost ON)
set(Stokhos_ENABLE_MATLABLib OFF)
set(Stokhos_ENABLE_MKL OFF)

# Exported cache variables
set(Stokhos_ENABLE_EXPLICIT_INSTANTIATION "ON")
set(HAVE_STOKHOS_EXPLICIT_INSTANTIATION "ON")
set(Stokhos_ENABLE_DEBUG "OFF")
set(STOKHOS_DEBUG "OFF")
set(Stokhos_ENABLE_TEUCHOS_TIME_MONITOR "ON")
set(STOKHOS_TEUCHOS_TIME_MONITOR "ON")
set(Stokhos_ENABLE_Experimental "OFF")
set(STOKHOS_EXPERIMENTAL "OFF")
set(Stokhos_ENABLE_Ensemble_Reduction "OFF")
set(HAVE_STOKHOS_ENSEMBLE_REDUCT "OFF")
set(Stokhos_ENABLE_Ensemble_GEMV "ON")
set(HAVE_STOKHOS_ENSEMBLE_GEMV "ON")
set(Stokhos_ENABLE_Ensemble_Scalar_Type "ON")
set(HAVE_STOKHOS_ENSEMBLE_SCALAR_TYPE "ON")
set(Stokhos_ENABLE_PCE_Scalar_Type "ON")
set(HAVE_STOKHOS_PCE_SCALAR_TYPE "ON")
set(Stokhos_ENABLE_OpenMP "OFF")
set(HAVE_STOKHOS_OPENMP "OFF")

# Include configuration of dependent packages
if (NOT TARGET Teuchos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Teuchos/TeuchosConfig.cmake")
endif()
if (NOT TARGET Kokkos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Kokkos/KokkosConfig.cmake")
endif()
if (NOT TARGET EpetraExt::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../EpetraExt/EpetraExtConfig.cmake")
endif()
if (NOT TARGET Ifpack::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Ifpack/IfpackConfig.cmake")
endif()
if (NOT TARGET ML::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ML/MLConfig.cmake")
endif()
if (NOT TARGET Anasazi::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Anasazi/AnasaziConfig.cmake")
endif()
if (NOT TARGET Sacado::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Sacado/SacadoConfig.cmake")
endif()
if (NOT TARGET NOX::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../NOX/NOXConfig.cmake")
endif()
if (NOT TARGET Isorropia::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Isorropia/IsorropiaConfig.cmake")
endif()
if (NOT TARGET KokkosKernels::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../KokkosKernels/KokkosKernelsConfig.cmake")
endif()
if (NOT TARGET TeuchosKokkosComm::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../TeuchosKokkosComm/TeuchosKokkosCommConfig.cmake")
endif()
if (NOT TARGET Tpetra::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Tpetra/TpetraConfig.cmake")
endif()
if (NOT TARGET Ifpack2::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Ifpack2/Ifpack2Config.cmake")
endif()
if (NOT TARGET MueLu::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../MueLu/MueLuConfig.cmake")
endif()
if (NOT TARGET Belos::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Belos/BelosConfig.cmake")
endif()
if (NOT TARGET Amesos2::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Amesos2/Amesos2Config.cmake")
endif()
if (NOT TARGET Thyra::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Thyra/ThyraConfig.cmake")
endif()
if (NOT TARGET Xpetra::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Xpetra/XpetraConfig.cmake")
endif()
if (NOT TARGET Boost::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../../external_packages/Boost/BoostConfig.cmake")
endif()

# Import Stokhos targets
include("${CMAKE_CURRENT_LIST_DIR}/StokhosTargets.cmake")

# Standard TriBITS-compliant external package variables
set(Stokhos_IS_TRIBITS_COMPLIANT TRUE)
set(Stokhos_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(Stokhos_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Stokhos_EXPORTED_PACKAGE_LIBS_NAMES "stokhos;stokhos_sacado;stokhos_tpetra_sd_mp_16_serial;stokhos_tpetra_mp_16_serial;stokhos_tpetraext_mp_16_serial;stokhos_tpetra_sd_pce_serial;stokhos_tpetra_pce_serial;stokhos_tpetraext_pce_serial;stokhos_tpetra;stokhos_xpetra_mp_16_serial;stokhos_xpetra_pce_serial;stokhos_xpetra;stokhos_amesos2;stokhos_ifpack2_mp_16_serial;stokhos_ifpack2_pce_serial;stokhos_ifpack2;stokhos_muelu_mp_16_serial;stokhos_muelu_pce_serial;stokhos_muelu")

foreach(libname IN LISTS Stokhos_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Stokhos::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Stokhos::${libname}', or better yet,"
      " 'Stokhos::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Stokhos'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Stokhos_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
