# @HEADER
# *****************************************************************************
#            TriBITS: Tribal Build, Integrate, and Test System
#
# Copyright 2013-2016 NTESS and the TriBITS contributors.
# SPDX-License-Identifier: BSD-3-Clause
# *****************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Thyra clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Thyra requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Thyra build
## ---------------------------------------------------------------------------

set(Thyra_CXX_COMPILER "/usr/bin/mpicxx")

set(Thyra_C_COMPILER "/usr/bin/mpicc")

set(Thyra_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(Thyra_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Thyra build
## ---------------------------------------------------------------------------

## Give the build type
set(Thyra_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(Thyra_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(Thyra_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=3 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection         -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer -flto=auto -Wno-incompatible-pointer-types -O3 -DNDEBUG]])

set(Thyra_Fortran_FLAGS [[ -O3]])
# Deprecated
set(Thyra_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(Thyra_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Thyra_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(Thyra_BUILD_SHARED_LIBS "ON")

set(Thyra_LINKER /usr/bin/ld)
set(Thyra_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Thyra_INSTALL_DIR "/usr")

## List of package libraries
set(Thyra_LIBRARIES Thyra::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Thyra_MPI_LIBRARIES "")
set(Thyra_MPI_LIBRARY_DIRS "")
set(Thyra_MPI_INCLUDE_DIRS "")
set(Thyra_MPI_EXEC "/usr/bin/mpiexec")
set(Thyra_MPI_EXEC_MAX_NUMPROCS "4")
set(Thyra_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Thyra_ENABLE_ThyraCore ON)
set(Thyra_ENABLE_ThyraEpetraAdapters ON)
set(Thyra_ENABLE_ThyraEpetraExtAdapters ON)
set(Thyra_ENABLE_ThyraTpetraAdapters ON)

# Exported cache variables
set(Thyra_ENABLE_DEBUG "OFF")
set(HAVE_THYRA_DEBUG "OFF")
set(Thyra_ENABLE_EXPLICIT_INSTANTIATION "ON")
set(HAVE_THYRA_EXPLICIT_INSTANTIATION "ON")
set(Thyra_ENABLE_ME_POLYNOMIAL "ON")
set(HAVE_THYRA_ME_POLYNOMIAL "ON")
set(Thyra_ENABLE_HESSIAN_SUPPORT "ON")
set(Thyra_BUILD_HESSIAN_SUPPORT "ON")

# Include configuration of dependent packages
if (NOT TARGET ThyraCore::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraCore/ThyraCoreConfig.cmake")
endif()
if (NOT TARGET ThyraEpetraAdapters::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraEpetraAdapters/ThyraEpetraAdaptersConfig.cmake")
endif()
if (NOT TARGET ThyraEpetraExtAdapters::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraEpetraExtAdapters/ThyraEpetraExtAdaptersConfig.cmake")
endif()
if (NOT TARGET ThyraTpetraAdapters::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../ThyraTpetraAdapters/ThyraTpetraAdaptersConfig.cmake")
endif()

# Import Thyra targets
include("${CMAKE_CURRENT_LIST_DIR}/ThyraTargets.cmake")

# Standard TriBITS-compliant external package variables
set(Thyra_IS_TRIBITS_COMPLIANT TRUE)
set(Thyra_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(Thyra_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Thyra_EXPORTED_PACKAGE_LIBS_NAMES "thyracore;ThyraCore_all_libs;thyraepetra;ThyraEpetraAdapters_all_libs;thyraepetraext;ThyraEpetraExtAdapters_all_libs;thyratpetra;ThyraTpetraAdapters_all_libs")

foreach(libname IN LISTS Thyra_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Thyra::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Thyra::${libname}', or better yet,"
      " 'Thyra::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Thyra'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Thyra_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
